@@ -1,6 +1,7 @@
-{-# LANGUAGE CPP              #-}
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE RankNTypes       #-}
+{-# LANGUAGE CPP                  #-}
+{-# LANGUAGE FlexibleContexts     #-}
+{-# LANGUAGE RankNTypes           #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Test.Hspec.Megaparsec.AdHoc
   ( -- * Helpers to run parsers
@@ -15,25 +16,33 @@ module Test.Hspec.Megaparsec.AdHoc
   , nes
     -- * Other
   , abcRow
-  , toFirstMismatch )
+  , toFirstMismatch
+  , rightOrder
+  , Parser )
 where
 
 import Control.Monad
 import Control.Monad.Reader
 import Control.Monad.Trans.Identity
-import Data.Foldable (foldl')
 import Data.List.NonEmpty (NonEmpty (..))
+import Data.Proxy
+import Data.Void
 import Test.Hspec
 import Test.Hspec.Megaparsec
-import Text.Megaparsec.Error
-import Text.Megaparsec.Pos
-import Text.Megaparsec.Prim
+import Test.QuickCheck
+import Text.Megaparsec
 import qualified Control.Monad.RWS.Lazy      as L
 import qualified Control.Monad.RWS.Strict    as S
 import qualified Control.Monad.State.Lazy    as L
 import qualified Control.Monad.State.Strict  as S
 import qualified Control.Monad.Writer.Lazy   as L
 import qualified Control.Monad.Writer.Strict as S
+import qualified Data.ByteString             as B
+import qualified Data.ByteString.Lazy        as BL
+import qualified Data.List.NonEmpty          as NE
+import qualified Data.Set                    as E
+import qualified Data.Text                   as T
+import qualified Data.Text.Lazy              as TL
 
 #if !MIN_VERSION_base(4,8,0)
 import Control.Applicative
@@ -46,20 +55,18 @@ import Control.Applicative
 -- that assumes empty file name.
 
 prs
-  :: Parsec Dec String a -- ^ Parser to run
+  :: Parser a          -- ^ Parser to run
   -> String            -- ^ Input for the parser
-  -> Either (ParseError Char Dec) a -- ^ Result of parsing
+  -> Either (ParseError Char Void) a -- ^ Result of parsing
 prs p = parse p ""
-{-# INLINE prs #-}
 
--- | Just like 'prs', but allows to inspect final state of the parser.
+-- | Just like 'prs', but allows to inspect the final state of the parser.
 
 prs'
-  :: Parsec Dec String a -- ^ Parser to run
+  :: Parser a          -- ^ Parser to run
   -> String            -- ^ Input for the parser
-  -> (State String, Either (ParseError Char Dec) a) -- ^ Result of parsing
+  -> (State String, Either (ParseError Char Void) a) -- ^ Result of parsing
 prs' p s = runParser' p (initialState s)
-{-# INLINE prs' #-}
 
 -- | Just like 'prs', but forces the parser to consume all input by adding
 -- 'eof':
@@ -67,19 +74,18 @@ prs' p s = runParser' p (initialState s)
 -- > prs_ p = parse (p <* eof) ""
 
 prs_
-  :: Parsec Dec String a -- ^ Parser to run
+  :: Parser a          -- ^ Parser to run
   -> String            -- ^ Input for the parser
-  -> Either (ParseError Char Dec) a -- ^ Result of parsing
+  -> Either (ParseError Char Void) a -- ^ Result of parsing
 prs_ p = parse (p <* eof) ""
-{-# INLINE prs_ #-}
 
 -- | Just like 'prs', but interprets given parser as various monads (tries
 -- all supported monads transformers in turn).
 
 grs
-  :: (forall m. MonadParsec Dec String m => m a) -- ^ Parser to run
+  :: (forall m. MonadParsec Void String m => m a) -- ^ Parser to run
   -> String            -- ^ Input for the parser
-  -> (Either (ParseError Char Dec) a -> Expectation)
+  -> (Either (ParseError Char Void) a -> Expectation)
     -- ^ How to check result of parsing
   -> Expectation
 grs p s r = do
@@ -93,12 +99,12 @@ grs p s r = do
   r (prs (evalRWSTL    p)    s)
   r (prs (evalRWSTS    p)    s)
 
--- | 'grs'' to 'grs' as 'prs'' to 'prs'.
+-- | 'grs'' to 'grs' is as 'prs'' to 'prs'.
 
 grs'
-  :: (forall m. MonadParsec Dec String m => m a) -- ^ Parser to run
+  :: (forall m. MonadParsec Void String m => m a) -- ^ Parser to run
   -> String            -- ^ Input for the parser
-  -> ((State String, Either (ParseError Char Dec) a) -> Expectation)
+  -> ((State String, Either (ParseError Char Void) a) -> Expectation)
     -- ^ How to check result of parsing
   -> Expectation
 grs' p s r = do
@@ -137,19 +143,12 @@ updatePosString
   -> SourcePos         -- ^ Initial position
   -> String            -- ^ 'String' — collection of tokens to process
   -> SourcePos         -- ^ Final position
-updatePosString w = foldl' f
-  where f p t = snd (defaultUpdatePos w p t)
-
--- | Position with minimal value.
-
-pos1 :: Pos
-pos1 = unsafePos 1
+updatePosString = advanceN (Proxy :: Proxy String)
 
 -- | Make a singleton non-empty list from a value.
 
 nes :: a -> NonEmpty a
 nes x = x :| []
-{-# INLINE nes #-}
 
 ----------------------------------------------------------------------------
 -- Other
@@ -171,3 +170,83 @@ toFirstMismatch
   -> String            -- ^ Resulting prefix
 toFirstMismatch f str s = take (n + 1) s
   where n = length (takeWhile (uncurry f) (zip str s))
+
+-- | Check that the given parser returns the list in the right order.
+
+rightOrder
+  :: Parser String     -- ^ The parser to test
+  -> String            -- ^ Input for the parser
+  -> String            -- ^ Expected result
+  -> Spec
+rightOrder p s s' =
+  it "produces the list in the right order" $
+    prs_ p s `shouldParse` s'
+
+-- | The type of parser that consumes a 'String'.
+
+type Parser = Parsec Void String
+
+----------------------------------------------------------------------------
+-- Arbitrary instances
+
+instance Arbitrary Void where
+  arbitrary = error "Arbitrary Void"
+
+instance Arbitrary Pos where
+  arbitrary = mkPos <$> (getSmall <$> arbitrary `suchThat` (> 0))
+
+instance Arbitrary SourcePos where
+  arbitrary = SourcePos
+    <$> sized (\n -> do
+          k <- choose (0, n `div` 2)
+          vectorOf k arbitrary)
+    <*> arbitrary
+    <*> arbitrary
+
+instance Arbitrary t => Arbitrary (ErrorItem t) where
+  arbitrary = oneof
+    [ Tokens <$> (NE.fromList . getNonEmpty <$> arbitrary)
+    , Label  <$> (NE.fromList . getNonEmpty <$> arbitrary)
+    , return EndOfInput ]
+
+instance Arbitrary (ErrorFancy a) where
+  arbitrary = oneof
+    [ sized (\n -> do
+        k <- choose (0, n `div` 2)
+        ErrorFail <$> vectorOf k arbitrary)
+    , ErrorIndentation <$> arbitrary <*> arbitrary <*> arbitrary ]
+
+instance (Arbitrary t, Ord t, Arbitrary e, Ord e)
+    => Arbitrary (ParseError t e) where
+  arbitrary = oneof
+    [ TrivialError
+      <$> (NE.fromList . getNonEmpty <$> arbitrary)
+      <*> arbitrary
+      <*> (E.fromList <$> arbitrary)
+    , FancyError
+      <$> (NE.fromList . getNonEmpty <$> arbitrary)
+      <*> (E.fromList <$> arbitrary) ]
+
+instance Arbitrary a => Arbitrary (State a) where
+  arbitrary = State
+    <$> arbitrary
+    <*> (NE.fromList . getNonEmpty <$> arbitrary)
+    <*> choose (1, 10000)
+    <*> (mkPos <$> choose (1, 20))
+
+instance Arbitrary T.Text where
+  arbitrary = T.pack <$> arbitrary
+
+instance Arbitrary TL.Text where
+  arbitrary = TL.pack <$> arbitrary
+
+instance Arbitrary B.ByteString where
+  arbitrary = B.pack <$> arbitrary
+
+instance Arbitrary BL.ByteString where
+  arbitrary = BL.pack <$> arbitrary
+
+#if MIN_VERSION_QuickCheck(2,10,0)
+instance Arbitrary a => Arbitrary (NonEmpty a) where
+  arbitrary = NE.fromList <$> (arbitrary `suchThat` (not . null))
+#endif
