@@ -11,9 +11,8 @@ import Test.Hspec
 import Test.Hspec.Megaparsec
 import Test.Hspec.Megaparsec.AdHoc
 import Test.QuickCheck
+import Text.Megaparsec
 import Text.Megaparsec.Char
-import Text.Megaparsec.Error
-import Text.Megaparsec.Prim
 
 #if !MIN_VERSION_base(4,8,0)
 import Control.Applicative
@@ -48,8 +47,7 @@ spec = do
       it "signals correct parse error" $
         property $ \ch -> ch /= '\n' ==> do
           let s = ['\r',ch]
-          prs eol s `shouldFailWith` err posI
-            (utoks s <> utok '\r' <> elabel "end of line")
+          prs eol s `shouldFailWith` err posI (utoks s <> elabel "end of line")
     context "when input stream is '\\r'" $
       it "signals correct parse error" $
         prs eol "\r" `shouldFailWith` err posI
@@ -59,7 +57,7 @@ spec = do
         property $ \ch s -> (ch `notElem` "\r\n") ==> do
           let s' = ch : s
           prs eol s' `shouldFailWith` err posI
-            (utok ch <> elabel "end of line")
+            (utoks (take 2 s') <> elabel "end of line")
     context "when stream is empty" $
       it "signals correct parse error" $
         prs eol "" `shouldFailWith` err posI
@@ -69,12 +67,31 @@ spec = do
     checkStrLit "tab" "\t" (pure <$> tab)
 
   describe "space" $
-    it "consumes it up to first non-space character" $
-      property $ \s -> do
-        let (s0,s1) = partition isSpace s
-            s' = s0 ++ s1
-        prs  space s' `shouldParse` ()
-        prs' space s' `succeedsLeaving` s1
+    it "consumes space up to first non-space character" $
+      property $ \s' -> do
+        let (s0,s1) = partition isSpace s'
+            s = s0 ++ s1
+        prs  space s `shouldParse` ()
+        prs' space s `succeedsLeaving` s1
+
+  describe "space1" $ do
+    context "when stream does not start with a space character" $
+      it "signals correct parse error" $
+        property $ \ch s' -> not (isSpace ch) ==> do
+          let (s0,s1) = partition isSpace s'
+              s = ch : s0 ++ s1
+          prs  space1 s `shouldFailWith` err posI (utok ch <> elabel "white space")
+          prs' space1 s `failsLeaving` s
+    context "when stream starts with a space character" $
+      it "consumes space up to first non-space character" $
+        property $ \s' -> do
+          let (s0,s1) = partition isSpace s'
+              s = ' ' : s0 ++ s1
+          prs  space1 s `shouldParse` ()
+          prs' space1 s `succeedsLeaving` s1
+    context "when stream is empty" $
+      it "signals correct parse error" $
+        prs space1 "" `shouldFailWith` err posI (ueof <> elabel "white space")
 
   describe "controlChar" $
     checkCharPred "control character" isControl controlChar
@@ -188,9 +205,11 @@ spec = do
           prs  (char ch) "" `shouldFailWith` err posI (ueof <> etok ch)
 
   describe "char'" $ do
+    let goodChar x =
+          (toUpper x == toLower x) || (isUpper x || isLower x)
     context "when stream begins with the character specified as argument" $
       it "parses the character" $
-        property $ \ch s -> do
+        property $ \ch s -> goodChar ch ==> do
           let sl = toLower ch : s
               su = toUpper ch : s
           prs  (char' ch) sl `shouldParse`     toLower ch
@@ -199,14 +218,14 @@ spec = do
           prs' (char' ch) su `succeedsLeaving` s
     context "when stream does not begin with the character specified as argument" $
       it "signals correct parse error" $
-        property $ \ch ch' s -> toLower ch /= toLower ch' ==> do
+        property $ \ch ch' s -> goodChar ch && toLower ch /= toLower ch' ==> do
           let s' = ch' : s
               ms = utok ch' <> etok (toLower ch) <> etok (toUpper ch)
           prs  (char' ch) s' `shouldFailWith` err posI ms
           prs' (char' ch) s' `failsLeaving`   s'
     context "when stream is empty" $
       it "signals correct parse error" $
-        property $ \ch -> do
+        property $ \ch -> goodChar ch ==> do
           let ms = ueof <> etok (toLower ch) <> etok (toUpper ch)
           prs  (char' ch) "" `shouldFailWith` err posI ms
 
@@ -221,6 +240,24 @@ spec = do
       it "signals correct parse error" $
         prs anyChar "" `shouldFailWith` err posI (ueof <> elabel "character")
 
+  describe "notChar" $ do
+    context "when stream begins with the character specified as argument" $
+      it "signals correct parse error" $
+        property $ \ch s' -> do
+          let p = notChar ch
+              s = ch : s'
+          prs p s `shouldFailWith` err posI (utok ch)
+          prs' p s `failsLeaving` s
+    context "when stream does not begin with the character specified as argument" $
+      it "parses first character in the stream" $
+        property $ \ch s -> not (null s) && ch /= head s ==> do
+          let p = notChar ch
+          prs  p s `shouldParse` head s
+          prs' p s `succeedsLeaving` tail s
+    context "when stream is empty" $
+      it "signals correct parse error" $
+        prs (notChar 'a') "" `shouldFailWith` err posI ueof
+
   describe "oneOf" $ do
     context "when stream begins with one of specified characters" $
       it "parses the character" $
@@ -241,29 +278,6 @@ spec = do
         property $ \chs ->
           prs (oneOf (chs :: String)) "" `shouldFailWith` err posI ueof
 
-  describe "oneOf'" $ do
-    context "when stream begins with one of specified characters" $
-      it "parses the character" $
-        property $ \chs' n s -> do
-          let chs = getNonEmpty chs'
-              ch  = chs !! (getNonNegative n `rem` length chs)
-              sl  = toLower ch : s
-              su  = toUpper ch : s
-          prs  (oneOf' chs) sl `shouldParse`     toLower ch
-          prs  (oneOf' chs) su `shouldParse`     toUpper ch
-          prs' (oneOf' chs) sl `succeedsLeaving` s
-          prs' (oneOf' chs) su `succeedsLeaving` s
-    context "when stream does not begin with any of specified characters" $
-      it "signals correct parse error" $
-        property $ \chs ch s  -> ch `notElemi` (chs :: String) ==> do
-          let s' = ch : s
-          prs  (oneOf' chs) s' `shouldFailWith` err posI (utok ch)
-          prs' (oneOf' chs) s' `failsLeaving`   s'
-    context "when stream is empty" $
-      it "signals correct parse error" $
-        property $ \chs ->
-          prs (oneOf' (chs :: String)) "" `shouldFailWith` err posI ueof
-
   describe "noneOf" $ do
     context "when stream does not begin with any of specified characters" $
       it "parses the character" $
@@ -284,29 +298,6 @@ spec = do
         property $ \chs ->
           prs (noneOf (chs :: String)) "" `shouldFailWith` err posI ueof
 
-  describe "noneOf'" $ do
-    context "when stream does not begin with any of specified characters" $
-      it "parses the character" $
-        property $ \chs ch s  -> ch `notElemi` (chs :: String) ==> do
-          let sl  = toLower ch : s
-              su  = toUpper ch : s
-          prs  (noneOf' chs) sl `shouldParse`     toLower ch
-          prs  (noneOf' chs) su `shouldParse`     toUpper ch
-          prs' (noneOf' chs) sl `succeedsLeaving` s
-          prs' (noneOf' chs) su `succeedsLeaving` s
-    context "when stream begins with one of specified characters" $
-      it "signals correct parse error" $
-        property $ \chs' n s -> do
-          let chs = getNonEmpty chs'
-              ch  = chs !! (getNonNegative n `rem` length chs)
-              s'  = ch : s
-          prs  (noneOf' chs) s' `shouldFailWith` err posI (utok ch)
-          prs' (noneOf' chs) s' `failsLeaving`   s'
-    context "when stream is empty" $
-      it "signals correct parse error" $
-        property $ \chs ->
-          prs (noneOf' (chs :: String)) "" `shouldFailWith` err posI ueof
-
   describe "string" $ do
     context "when stream is prefixed with given string" $
       it "parses the string" $
@@ -317,9 +308,9 @@ spec = do
     context "when stream is not prefixed with given string" $
       it "signals correct parse error" $
         property $ \str s -> not (str `isPrefixOf` s) ==> do
-          let n = length (takeWhile (uncurry (==)) (zip str s)) + 1
-              common = take n s
-          prs  (string str) s `shouldFailWith` err posI (utoks common <> etoks str)
+          let us = take (length str) s
+          prs (string str) s `shouldFailWith`
+            err posI (utoks us <> etoks str)
 
   describe "string'" $ do
     context "when stream is prefixed with given string" $
@@ -332,14 +323,14 @@ spec = do
     context "when stream is not prefixed with given string" $
       it "signals correct parse error" $
         property $ \str s -> not (str `isPrefixOfI` s) ==> do
-          let n = length (takeWhile (uncurry casei) (zip str s)) + 1
-              common = take n s
-          prs  (string' str) s `shouldFailWith` err posI (utoks common <> etoks str)
+          let us = take (length str) s
+          prs  (string' str) s `shouldFailWith`
+            err posI (utoks us <> etoks str)
 
 ----------------------------------------------------------------------------
 -- Helpers
 
-checkStrLit :: String -> String -> Parsec Dec String String -> SpecWith ()
+checkStrLit :: String -> String -> Parser String -> SpecWith ()
 checkStrLit name ts p = do
   context ("when stream begins with " ++ name) $
     it ("parses the " ++ name) $
@@ -351,13 +342,14 @@ checkStrLit name ts p = do
     it "signals correct parse error" $
       property $ \ch s -> ch /= head ts ==> do
        let s' = ch : s
-       prs  p s' `shouldFailWith` err posI (utok ch <> etoks ts)
+           us = take (length ts) s'
+       prs  p s' `shouldFailWith` err posI (utoks us <> etoks ts)
        prs' p s' `failsLeaving`   s'
   context "when stream is empty" $
     it "signals correct parse error" $
       prs p "" `shouldFailWith` err posI (ueof <> etoks ts)
 
-checkCharPred :: String -> (Char -> Bool) -> Parsec Dec String Char -> SpecWith ()
+checkCharPred :: String -> (Char -> Bool) -> Parser Char -> SpecWith ()
 checkCharPred name f p = do
   context ("when stream begins with " ++ name) $
     it ("parses the " ++ name) $
@@ -375,7 +367,7 @@ checkCharPred name f p = do
     it "signals correct parse error" $
       prs p "" `shouldFailWith` err posI (ueof <> elabel name)
 
-checkCharRange :: String -> String -> Parsec Dec String Char -> SpecWith ()
+checkCharRange :: String -> String -> Parser Char -> SpecWith ()
 checkCharRange name tchs p = do
   forM_ tchs $ \tch ->
     context ("when stream begins with " ++ showTokens (nes tch)) $
@@ -384,12 +376,6 @@ checkCharRange name tchs p = do
           let s' = tch : s
           prs  p s' `shouldParse`     tch
           prs' p s' `succeedsLeaving` s
-  -- context ("when stream does not begin with " ++ name) $
-  --   it "signals correct parse error" $
-  --     property $ \ch s -> ch `notElem` tchs ==> do
-  --      let s' = ch : s
-  --      prs  p s' `shouldFailWith` err posI (utok ch <> elabel name)
-  --      prs' p s' `failsLeaving`   s'
   context "when stream is empty" $
     it "signals correct parse error" $
       prs p "" `shouldFailWith` err posI (ueof <> elabel name)
@@ -398,24 +384,15 @@ checkCharRange name tchs p = do
 
 fuzzyCase :: String -> Gen String
 fuzzyCase s = zipWith f s <$> vector (length s)
-  where f k True  = if isLower k then toUpper k else toLower k
-        f k False = k
+  where
+    f k True  = if isLower k then toUpper k else toLower k
+    f k False = k
 
 -- | Case-insensitive equality test for characters.
 
 casei :: Char -> Char -> Bool
 casei x y = toUpper x == toUpper y
 
--- | Case-insensitive 'elem'.
-
-elemi :: Char -> String -> Bool
-elemi c = any (casei c)
-
--- | Case-insensitive 'notElem'.
-
-notElemi :: Char -> String -> Bool
-notElemi c = not . elemi c
-
 -- | The 'isPrefixOf' function takes two 'String's and returns 'True' iff
 -- the first list is a prefix of the second with case-insensitive
 -- comparison.
