@@ -4,19 +4,17 @@
 
 module Text.Megaparsec.ExprSpec (spec) where
 
-import Control.Applicative (some, (<|>))
 import Data.Monoid ((<>))
 import Test.Hspec
 import Test.Hspec.Megaparsec
 import Test.Hspec.Megaparsec.AdHoc
 import Test.QuickCheck
+import Text.Megaparsec
 import Text.Megaparsec.Char
-import Text.Megaparsec.Combinator
 import Text.Megaparsec.Expr
-import Text.Megaparsec.Prim
 
 #if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>), (<*), (<*>), (*>), pure)
+import Control.Applicative hiding (many, some)
 #endif
 
 spec :: Spec
@@ -34,9 +32,8 @@ spec =
     context "when term is missing" $
       it "signals correct parse error" $ do
         let p = expr <* eof
-            n = 1 :: Integer
-        prs p "-" `shouldFailWith` err (posN n "-") (ueof <> elabel "term")
-        prs p "(" `shouldFailWith` err (posN n "(") (ueof <> elabel "term")
+        prs p "-" `shouldFailWith` err (posN 1 "-") (ueof <> elabel "term")
+        prs p "(" `shouldFailWith` err (posN 1 "(") (ueof <> elabel "term")
         prs p "*" `shouldFailWith` err posI (utok '*' <> elabel "term")
     context "operator is missing" $
       it "signals correct parse error" $
@@ -109,7 +106,8 @@ instance Arbitrary Node where
 arbitraryN0 :: Int -> Gen Node
 arbitraryN0 n = frequency [ (1, Mod <$> leaf <*> leaf)
                           , (9, arbitraryN1 n) ]
-  where leaf = arbitraryN1 (n `div` 2)
+  where
+    leaf = arbitraryN1 (n `div` 2)
 
 arbitraryN1 :: Int -> Gen Node
 arbitraryN1 n =
@@ -120,21 +118,22 @@ arbitraryN1 n =
 arbitraryN2 :: Int -> Gen Node
 arbitraryN2 0 = Val . getNonNegative <$> arbitrary
 arbitraryN2 n = elements [Sum,Sub,Pro,Div,Exp] <*> leaf <*> leaf
-  where leaf = arbitraryN0 (n `div` 2)
+  where
+    leaf = arbitraryN0 (n `div` 2)
 
 -- Some helpers are put here since we don't want to depend on
 -- "Text.Megaparsec.Lexer".
 
-lexeme :: (MonadParsec e s m, Token s ~ Char) => m a -> m a
+lexeme :: Parser a -> Parser a
 lexeme p = p <* hidden space
 
-symbol :: (MonadParsec e s m, Token s ~ Char) => String -> m String
+symbol :: String -> Parser String
 symbol = lexeme . string
 
-parens :: (MonadParsec e s m, Token s ~ Char) => m a -> m a
+parens :: Parser a -> Parser a
 parens = between (symbol "(") (symbol ")")
 
-integer :: (MonadParsec e s m, Token s ~ Char) => m Integer
+integer :: Parser Integer
 integer = lexeme (read <$> some digitChar <?> "integer")
 
 -- Here we use a table of operators that makes use of all features of
@@ -142,18 +141,19 @@ integer = lexeme (read <$> some digitChar <?> "integer")
 -- but valid expressions and render them to get their textual
 -- representation.
 
-expr :: (MonadParsec e s m, Token s ~ Char) => m Node
+expr :: Parser Node
 expr = makeExprParser term table
 
-term :: (MonadParsec e s m, Token s ~ Char) => m Node
+term :: Parser Node
 term = parens expr <|> (Val <$> integer) <?> "term"
 
-table :: (MonadParsec e s m, Token s ~ Char) => [[Operator m Node]]
-table = [ [ Prefix  (symbol "-" *> pure Neg)
-          , Postfix (symbol "!" *> pure Fac)
-          , InfixN  (symbol "%" *> pure Mod) ]
-        , [ InfixR  (symbol "^" *> pure Exp) ]
-        , [ InfixL  (symbol "*" *> pure Pro)
-          , InfixL  (symbol "/" *> pure Div) ]
-        , [ InfixL  (symbol "+" *> pure Sum)
-          , InfixL  (symbol "-" *> pure Sub)] ]
+table :: [[Operator Parser Node]]
+table =
+  [ [ Prefix  (Neg <$ symbol "-")
+    , Postfix (Fac <$ symbol "!")
+    , InfixN  (Mod <$ symbol "%") ]
+  , [ InfixR  (Exp <$ symbol "^") ]
+  , [ InfixL  (Pro <$ symbol "*")
+    , InfixL  (Div <$ symbol "/") ]
+  , [ InfixL  (Sum <$ symbol "+")
+    , InfixL  (Sub <$ symbol "-")] ]
